/*
  STATUS OK
  Version 2.2017_07_27
  Kompiliert unter Arduino 1.8.3

  1. Unter Verwendung der mit Arduino 1.8.3 mitglieferten libraries:
  Wire
  ...\arduino-1.8.3\hardware\arduino\avr\libraries\Wire\src\Wire.cpp
  ...\arduino-1.8.3\hardware\arduino\avr\libraries\Wire\src\Wire.h
  LiquidCrystal:
  ...\arduino-1.8.3\libraries\LiquidCrystal\src\LiquidCrystal.cpp
  ...\arduino-1.8.3\libraries\LiquidCrystal\src\LiquidCrystal.h

  2. Zusaetzliche Libraries (https://www.i2cdevlib.com) im
  libraries Verzeichnis innerhalb des Sketch Verzeichnis:

  ADXL345
  ...\ARDUINO_SKETCHES\libraries\ADXL345\
  ...\ARDUINO_SKETCHES\libraries\ADXL345\ADXL345.cpp
  ...\ARDUINO_SKETCHES\libraries\ADXL345\ADXL345.h

  I2Cdev
  ...\ARDUINO_SKETCHES\libraries\I2Cdev\
  ...\ARDUINO_SKETCHES\libraries\I2Cdev\I2Cdev.cpp
  ...\ARDUINO_SKETCHES\libraries\I2Cdev\I2Cdev.h

  3. Verwendete Hardware, die bei mir funktioniert
  3.1 LCD KeypadShield
  3.2 Arduino UNO R3
  3.3 ADXL345 BreakoutBoard, GY-291
*/
/*******************************************************************************************/
// fuer den Betrieb von zwei ADXL Sensoren an einem Arduino eignet sich diese Library, zusammen mit I2Cdev
// https://www.i2cdevlib.com

// https://www.i2cdevlib.com/usage
// Arduino Wire library is required if I2Cdev I2CDEV_ARDUINO_WIRE implementation
// is used in I2Cdev.h
#include <Wire.h>

// https://www.i2cdevlib.com/usage
// I2Cdev and ADXL345 must be installed as libraries, or else the .cpp/.h files
// for both classes must be in the include path of your project
#include <ADXL345.h>
ADXL345 adxl1(0x53);  // Default I2C Adresse, mit Pin SDO nicht beschaltet, interner PullDown
ADXL345 adxl2(0x1D);  // Zweite  I2C Adresse, mit Pin SDO an VCC
/*******************************************************************************************/
//Fuer das LCD KeyPad-Shield
#include <LiquidCrystal.h>    // für das LCD
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
/*******************************************************************************************/
// define some values used by the LCD panel and KeypadShield buttons
int lcd_key     = 0;
int adc_key_in  = 0;
/************************************ DEFINED CONSTANTS ************************************/
#define btnRIGHT  0
#define btnUP     1
#define btnDOWN   2
#define btnLEFT   3
#define btnSELECT 4
#define btnNONE   5
/************************************ DEFINED FUNCTIONS ************************************/
int read_LCD_buttonsFunction()
{
  adc_key_in = analogRead(0);      // read the value from the sensor / KeypadShield
  // the values depends on KeypadShield, use
  if (adc_key_in > 1000) return btnNONE;
  if (adc_key_in < 50)   return btnRIGHT;
  if (adc_key_in < 150)  return btnUP;
  if (adc_key_in < 300)  return btnDOWN  ;
  if (adc_key_in < 500)  return btnLEFT;
  if (adc_key_in < 700)  return btnSELECT;
  return btnNONE;
}
/************************************ DEFINED FUNCTIONS ************************************/
int toggle = LOW;        // variable for toggling between THROW and SECANT
int count = 30;           // Start chord depth [mm]

boolean serialDebug = false; // usefull using adruino uno, does not work on other arduino boards
boolean skip = false;        // I already know usage
/************************************ DEFINED VARIABLES ************************************/
// define some values used by angle calculation
int x1, y1, z1;                 // raw values from sensor (-255 ... 0 ... +255)
int x2, y2, z2;                 // raw values from sensor (-255 ... 0 ... +255)
int angle1;
int angle2;
int throw1;
int throw2;
int secant1;
int secant2;
int up1;                        // 1 -> up, 2 -> down
int up2;                        // 1 -> up, 2 -> down
int relative1;                  // for zeroing
int relative2;                  // for zeroing
float rad1;
float rad2;
float x1g, y1g, z1g;            // g values
float x2g, y2g, z2g;            // g values
float calc1;
float tilt1;
float calc2;
float tilt2;
/************************************ SENSORCALIBRATION ************************************/
#define offsetX1   0       // OFFSET values
#define offsetY1   0       // replace "0" with calculated value after sensor calibration
#define offsetZ1   0
#define offsetX2   0       // OFFSET values
#define offsetY2   0       // replace "0" with calculated value after sensor calibration
#define offsetZ2   0

#define gainX1     1       // GAIN factors
#define gainY1     1       // replace "1" with calculated value after sensor calibration
#define gainZ1     1
#define gainX2     1       // GAIN factors
#define gainY2     1       // replace "1" with calculated value after sensor calibration
#define gainZ2     1
/************************************ Grapics for display **********************************/
uint8_t degree[8]  = {140, 146, 146, 140, 128, 128, 128, 128};      // °
uint8_t arrowup[8]  = {132, 142, 132, 132, 132, 128, 128, 128};     // Arrow up
uint8_t arrowdown[8]  = {128, 128, 128, 132, 132, 132, 142, 132,};  // Arrow down
/************************************** FUNCTION SETUP *************************************/
void setup() {
  // https://www.i2cdevlib.com/usage
  // join I2C bus (I2Cdev library doesn't do this automatically)
  Wire.begin();

  // power on , initialize acceleration sensors
  adxl1.initialize();
  adxl2.initialize();

  // power on LCD
  lcd.begin (16, 2);            // for 16 x 2 LCD module

  lcd.createChar(0, degree);
  lcd.createChar(1, arrowup);
  lcd.createChar(2, arrowdown);
  /*******************************************************************************************/
  // START SCREEN
  /*******************************************************************************************/

  if (skip = false) {
    lcd.home ();
    lcd.print("RC - METER");
    lcd.setCursor (0, 1);
    lcd.print("ANGLE AND THROW");
    delay(3000);
    lcd.clear ();
    lcd.print("RC - METER");
    lcd.setCursor (0, 1);
    lcd.print("  >>  OR  <<");
    delay(500);
    lcd.clear ();
    lcd.print("RC - METER");
    lcd.setCursor (0, 1);
    lcd.print("ANGLE AND SECANT");
    delay(3000);

    lcd.clear ();
    lcd.print("to ZERO, press:");
    lcd.setCursor (0, 1);
    lcd.print("Key SELECT");
    delay(3000);

    lcd.clear ();
    lcd.print("set CORD depth..");
    delay(1000);
    lcd.setCursor (0, 0);
    lcd.print(".in [mm], press:");
    lcd.setCursor (0, 1);
    lcd.print("Key UP or DOWN");
    delay(3000);

    lcd.clear ();
    lcd.print("choose THROW or.");
    delay(1000);
    lcd.setCursor (0, 0);
    lcd.print("..SECANT, press:");
    lcd.setCursor (0, 1);
    lcd.print("Key LEFT");
    delay(3000);

    lcd.clear ();
    lcd.print("to RESET, press:");
    lcd.setCursor (0, 1);
    lcd.print("Key RST");
    delay(3000);
  }
  else {}

  if (serialDebug) {
    Serial.print("Initialize Serial Port...");
    Serial.begin(9600);
    Serial.println("OK");
    delay(1000);
  }

}
/**************************************** FUNCTION MAIN LOOP *******************************/
void loop() {
  lcd_key = read_LCD_buttonsFunction();

  switch (lcd_key)               // depending on which button was pushed, we perform an action
  {
    case btnSELECT:
      {
        relative1 = angle1;
        relative2 = angle2;
        if (serialDebug) {
          Serial.println(lcd_key);
        }
        else {}
        break;
      }
    case btnUP:
      {
        count = count + 1;
        if (serialDebug) {
          Serial.println(lcd_key);
        }
        else {}
        break;
      }
    case btnDOWN:
      {
        if  (count > 1) {
          count = count - 1;
          if (serialDebug) {
            Serial.println(lcd_key);
          }
          else {}

        }
        break;
      }
    case btnLEFT:
      {
        if (toggle == HIGH) {
          toggle = LOW;
        }
        else {
          toggle = HIGH;
        }
        if (serialDebug) {
          Serial.print(lcd_key);
          Serial.print(" ");
          Serial.println(toggle);
        }
        else {}
        delay (300);
        break;
      }
    case btnRIGHT:
      {
        if (serialDebug) {
          Serial.println(lcd_key);
        }
        else {}
        break;
      }
    default:
      {
        if (serialDebug) {
          Serial.print(lcd_key);
          Serial.print("--");
          Serial.println(count);
        }
        else {}
        break;
      }
  }

  adxl1.getAcceleration(&x1, &y1, &z1); // read the accelerometer values and store them in variables
  x1g = 0.00385 * (x1 - offsetX1) / gainX1; // 0.00385 to convert the raw data to Gs
  y1g = 0.00385 * (y1 - offsetY1) / gainY1; // offset and gain for calibration
  z1g = 0.00385 * (z1 - offsetZ1) / gainZ1;

  adxl2.getAcceleration(&x2, &y2, &z2); // read the accelerometer values and store them in variables
  x2g = 0.00385 * (x2 - offsetX2) / gainX2; // 0.00385 to convert the raw data to Gs
  y2g = 0.00385 * (y2 - offsetY2) / gainY2; // offset and gain for calibration
  z2g = 0.00385 * (z2 - offsetZ2) / gainZ2;

  calc1 = x1g / sqrt ((sq(y1g) + sq(z1g))); //sensor orientation!
  tilt1 = atan(calc1) * 57.296;
  angle1 = int(tilt1) * -1;               //sensor orientation!

  calc2 = x2g / sqrt ((sq(y2g) + sq(z2g)));  //sensor orientation!
  tilt2 = atan(calc2) * 57.296;
  angle2 = int(tilt2) * -1;               //sensor orientation!

  if ((angle1 - relative1) > 0) {
    up1 = 1;                     //arrow ↑↑↑
  }
  else {
    up1 = 2;                     //arrow ↓↓↓
  }
  if ((angle2 - relative2) > 0) {
    up2 = 1;                     //arrow ↑↑↑
  }
  else {
    up2 = 2;                     //arrow ↓↓↓
  }

  /*******************************************************************************************/
  lcd.setCursor (0, 0);         // go to start of 1st line
  lcd.clear();

  lcd.print("ANG");

  lcd.setCursor (4, 0);
  if ((angle1 - relative1) == 0) {
    lcd.print("0");
    lcd.setCursor (14, 0);
    lcd.write(byte(0));
  }
  else {
      lcd.print(abs(angle1 - relative1)); lcd.write(byte(up1));
      lcd.setCursor (14, 0);
      lcd.write(byte(0));
  }

  lcd.setCursor (8, 0);
  if ((angle2 - relative2) == 0) {
    lcd.print("0");
    lcd.setCursor (14, 0);
    lcd.write(byte(0));
  }
  else {
      lcd.print(abs(angle2 - relative2)); lcd.write(byte(up2));
      lcd.setCursor (14, 0);
      lcd.write(byte(0));
  }
  /*******************************************************************************************/
  lcd.setCursor (0, 1);                                     // go to start of 2nd line
  rad1 = abs(angle1 - relative1) * 0.017453;
  rad2 = abs(angle2 - relative2) * 0.017453;

  throw1 = int(count * sin(rad1));
  throw2 = int(count * sin(rad2));

  secant1 = int(2 * count * sin(rad1 / 2));
  secant2 = int(2 * count * sin(rad2 / 2));

  switch (lcd_key)               // depending on which button was pushed, we perform an action
  {
    case  btnUP: {
        lcd.clear();
        lcd.setCursor (0, 1);           // go to start of 1st line
        lcd.print("CHORD DEPTH:"); lcd.print(count); lcd.print("mm ");

        if (serialDebug) {
          Serial.print(lcd_key);
          Serial.println(" ");
          Serial.print("CHORD DEPTH:");
          Serial.print(count);
          Serial.println("mm");
        }
        else {}
        break;
      }
    case btnDOWN: {
        if  (count > 1) {
          lcd.clear();
          lcd.setCursor (0, 1);           // go to start of 1st line
          lcd.print("CHORD DEPTH:"); lcd.print(count); lcd.print("mm ");
        }
        else {}
        if (serialDebug) {
          Serial.print(lcd_key);
          Serial.println(" ");
          Serial.print("CHORD DEPTH:");
          Serial.print(count);
          Serial.println("mm");
        }
        else {}
        break;
      }
    default: {
        // read the state of the jumper value:
        if (toggle == HIGH) {
          lcd.print("SEC");
          lcd.setCursor (10, 1);
          lcd.print("("); lcd.print(count); lcd.print(")");
          lcd.setCursor (14, 1);
          lcd.print("mm");

          lcd.setCursor (4, 1);
          if ((secant1) == 0) {
            lcd.print("0");
          }
          else {
              lcd.print(secant1);
          }

          lcd.setCursor (8, 1);
          if ((secant2) == 0) {
            lcd.print("0");
          }
          else {
              lcd.print(secant2);
          }
        }
        else {
          lcd.print("THR");
          lcd.setCursor (10, 1);
          lcd.print("("); lcd.print(count); lcd.print(")");
          lcd.setCursor (14, 1);
          lcd.print("mm");

          lcd.setCursor (4, 1);
          if ((throw1) == 0) {
            lcd.print("0");
          }
          else {
              lcd.print(throw1);
          }

          lcd.setCursor (8, 1);
          if ((throw2) == 0) {
            lcd.print("0");
          }
          else {
              lcd.print(throw2);
          }
        }
        break;
      }
  }
  delay(300);
}

